# -*- python -*-
# $RCSfile: skeletoninfodisplay.py,v $
# $Revision: 1.13 $
# $Author: langer $
# $Date: 2005/05/13 18:44:10 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from oof2.common import color
from oof2.common import primitives
from oof2.common import registeredclass
from oof2.common.IO import display
from oof2.common.IO import parameter
from oof2.common.IO import xmlmenudump

class SkeletonInfoDisplay(display.DisplayMethod):
    def __init__(self, query_color, peek_color, node_size,
                 element_width, segment_width):
        self.query_color = query_color
        self.peek_color = peek_color
        self.colors = {"query": self.query_color, "peek": self.peek_color}
        self.node_size = node_size
        self.element_width = element_width
        self.segment_width = segment_width
        display.DisplayMethod.__init__(self)
        self.drawFuncs = {"Element": self.drawElement,
                          "Segment": self.drawSegment,
                          "Node": self.drawNode}

    def draw(self, gfxwindow, device):
        toolbox = gfxwindow.getToolboxByName("Skeleton_Info")
        # Drawing "queried" item.
        if toolbox.querier and toolbox.querier.object:
            self.drawFuncs[toolbox.querier.targetname](device,
                                                       toolbox.querier.object,
                                                       which="query")
        # Drawing "peeked" item.
        if toolbox.peeker and toolbox.peeker.objects.values():
            for objtype in toolbox.peeker.objects.keys():
                if toolbox.peeker.objects[objtype]:
                    self.drawFuncs[objtype](device,
                                            toolbox.peeker.objects[objtype],
                                            which="peek")

    def drawElement(self, device, element, which="query"):
        device.set_lineColor(self.colors[which])
        device.set_lineWidth(self.element_width)
        for i in range(element.nnodes()):
            n0 = element.nodes[i]
            n1 = element.nodes[(i+1)%element.nnodes()]
            device.draw_segment(primitives.Segment(n0.position(),
                                                   n1.position()))

    def drawSegment(self, device, segment, which="query"):
        device.set_lineColor(self.colors[which])
        device.set_lineWidth(self.segment_width)
        device.draw_segment(primitives.Segment(segment.nodes()[0].position(),
                                               segment.nodes()[1].position()))

    def drawNode(self, device, node, which="query"):
        device.set_lineColor(self.colors[which])
        device.set_lineWidth(self.node_size)
        device.draw_dot(node.position())

    def getTimeStamp(self, gfxwindow):
        toolbox = gfxwindow.getToolboxByName("Skeleton_Info")
        if toolbox.querier and toolbox.peeker:
            return max(self.timestamp,
                       toolbox.querier.getTimeStamp(),
                       toolbox.peeker.getTimeStamp())
        elif toolbox.querier and not toolbox.peeker:
            return max(self.timestamp,
                       toolbox.querier.getTimeStamp())
        else:
            return self.timestamp
                
# This object should be created via the registration, and not
# directly via the initializer, because the registration creation
# method gives it a timestamp.

skeletonInfoDisplay = registeredclass.Registration(
    'Info',
    display.DisplayMethod,
    SkeletonInfoDisplay,
    params=[color.ColorParameter('query_color', color.RGBColor(0.0, 0.5, 1.0),
                                 tip="Color for the queried objects."),
            color.ColorParameter('peek_color', color.RGBColor(1.0, 0.5, 0.5),
                                 tip="Color for the peeked objects."),
            parameter.IntRangeParameter('node_size', (0,10), 3,
                                        tip="Node size."),
            parameter.IntRangeParameter('element_width', (0,10), 3,
                                        tip="Line width for elements."),
            parameter.IntRangeParameter('segment_width', (0,10), 3,
                                        tip="Line width for segments.")],
    ordering=4.0,
    layerordering=display.SemiLinear(3),
    whoclasses=('Skeleton',),
    tip="Set parameters for the decorations used by the Skeleton Info toolbox.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/skeletoninfodisplay.xml')
    )

display.predefineLayer('Skeleton', '<topmost>', skeletonInfoDisplay())
